
#getAnywhere(summary.glm)
#getAnywhere(summary)
#T1 <- summary(M1, print=FALSE)
#MySummary.glm(T1) #Use: summary(M1)   

MySummary.glm <- function (x, digits = max(3L, getOption("digits") - 3L), symbolic.cor = x$symbolic.cor, 
                           signif.stars = getOption("show.signif.stars"), ...) 
{
  #cat("\nCall:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"), 
  #    "\n\n", sep = "")
  #cat("Deviance Residuals: \n")
  #if (x$df.residual > 5) {
  #    x$deviance.resid <- setNames(quantile(x$deviance.resid, 
  #        na.rm = TRUE), c("Min", "1Q", "Median", "3Q", "Max"))
  #}
  #xx <- zapsmall(x$deviance.resid, digits + 1L)
  #print.default(xx, digits = digits, na.print = "", print.gap = 2L)
  if (length(x$aliased) == 0L) {
    cat("\nNo Coefficients\n")
  }
  else {
    df <- if ("df" %in% names(x)) 
      x[["df"]]
    else NULL
    if (!is.null(df) && (nsingular <- df[3L] - df[1L])) 
      cat("\nCoefficients: (", nsingular, " not defined because of singularities)\n", 
          sep = "")
    else cat("\nCoefficients:\n")
    coefs <- x$coefficients
    if (!is.null(aliased <- x$aliased) && any(aliased)) {
      cn <- names(aliased)
      coefs <- matrix(NA, length(aliased), 4L, dimnames = list(cn, 
                                                               colnames(coefs)))
      coefs[!aliased, ] <- x$coefficients
    }
    printCoefmat(coefs, digits = digits, signif.stars = signif.stars, 
                 na.print = "NA", ...)
  }
  cat("\n(Dispersion parameter for ", x$family$family, " family taken to be ", 
      format(x$dispersion), ")\n\n", apply(cbind(paste(format(c("Null", 
                                                                "Residual"), justify = "right"), "deviance:"), format(unlist(x[c("null.deviance", 
                                                                                                                                 "deviance")]), digits = max(5L, digits + 1L)), " on", 
                                                 format(unlist(x[c("df.null", "df.residual")])), " degrees of freedom\n"), 
                                           1L, paste, collapse = " "), sep = "")
  if (nzchar(mess <- naprint(x$na.action))) 
    cat("  (", mess, ")\n", sep = "")
  cat("AIC: ", format(x$aic, digits = max(4L, digits + 1L)), 
      #"\n\n", "Number of Fisher Scoring iterations: ", x$iter, 
      "\n", sep = "")
  correl <- x$correlation
  if (!is.null(correl)) {
    p <- NCOL(correl)
    if (p > 1) {
      cat("\nCorrelation of Coefficients:\n")
      if (is.logical(symbolic.cor) && symbolic.cor) {
        print(symnum(correl, abbr.colnames = NULL))
      }
      else {
        correl <- format(round(correl, 2L), nsmall = 2L, 
                         digits = digits)
        correl[!lower.tri(correl)] <- ""
        print(correl[-1, -p, drop = FALSE], quote = FALSE)
      }
    }
  }
  cat("\n")
  invisible(x)
}



MytestDispersion <- function (simulationOutput, alternative = c("two.sided", "greater", 
                                            "less"), plot = T, type = c("DHARMa", "PearsonChisq"), ...) 
{
  alternative <- match.arg(alternative)
  type <- match.arg(type)
  out = list()
  out$data.name = deparse(substitute(simulationOutput))
  simulationOutput = DHARMa:::ensureDHARMa(simulationOutput, convert = "Model")
  if (type == "DHARMa") {
    if (simulationOutput$refit == F) {
      expectedVar = sd(simulationOutput$simulatedResponse)^2
      spread <- function(x) var(x - simulationOutput$fittedPredictedResponse)/expectedVar
      out = testGeneric(simulationOutput, summary = spread, 
                        alternative = alternative, methodName = "DHARMa nonparametric dispersion test", 
                        plot = plot, ...)
      names(out$statistic) = "dispersion"
    }
    else {
      observed = tryCatch(sum(residuals(simulationOutput$fittedModel, 
                                        type = "pearson")^2), error = function(e) {
                                          message(paste("DHARMa: the requested tests requires pearson residuals, but your model does not implement these calculations. Test will return NA. Error message:", 
                                                        e))
                                          return(NA)
                                        })
      if (is.na(observed)) 
        return(NA)
      expected = apply(simulationOutput$refittedPearsonResiduals^2, 
                       2, sum)
      out$statistic = c(dispersion = observed/mean(expected))
      names(out$statistic) = "dispersion"
      out$method = "DHARMa nonparametric dispersion test via mean deviance residual fitted vs. simulated-refitted"
      p = getP(simulated = expected, observed = observed, 
               alternative = alternative)
      out$alternative = alternative
      out$p.value = p
      class(out) = "htest"
      if (plot == T) {
        xLabel = paste("Simulated values, red line = fitted model. p-value (", 
                       out$alternative, ") = ", out$p.value, sep = "")
        hist(expected, xlim = range(expected, observed, 
                                    na.rm = T), col = "lightgrey", main = "", xlab = xLabel, 
             breaks = 20, cex.main = 1)
        abline(v = observed, lwd = 2, col = "red")
        main = ifelse(out$p.value <= 0.05, "Dispersion test significant", 
                      "Dispersion test n.s.")
        title(main = main, cex.main = 1, col.main = ifelse(out$p.value <= 
                                                             0.05, "red", "black"))
      }
    }
  }
  else if (type == "PearsonChisq") {
    if (!alternative == "greater") 
      message("Note that the chi2 test on Pearson residuals is biased for mixed models towards underdispersion. Tests with alternative = two.sided or less are therefore not reliable. I recommend to test only with alternative = 'greater', i.e. test for overdispersion")
    rdf <- df.residual(simulationOutput$fittedModel)
    rp <- residuals(simulationOutput$fittedModel, type = "pearson")
    Pearson.chisq <- sum(rp^2)
    prat <- Pearson.chisq/rdf
    if (alternative == "greater") 
      pval <- pchisq(Pearson.chisq, df = rdf, lower.tail = FALSE)
    else if (alternative == "less") 
      pval <- pchisq(Pearson.chisq, df = rdf, lower.tail = TRUE)
    else if (alternative == "two.sided") 
      pval <- min(min(pchisq(Pearson.chisq, df = rdf, lower.tail = TRUE), 
                      pchisq(Pearson.chisq, df = rdf, lower.tail = FALSE)) * 
                    2, 1)
    out$statistic = prat
    names(out$statistic) = "dispersion"
    out$parameter = rdf
    names(out$parameter) = "df"
    out$method = "Parametric dispersion test via mean Pearson-chisq statistic"
    out$alternative = alternative
    out$p.value = pval
    class(out) = "htest"
    return(out)
  }
  return(out)
}




MytestZeroInflation <- function (simulationOutput, ...) 
{
  countZeros <- function(x) sum(x == 0)
  testGeneric(simulationOutput = simulationOutput, summary = countZeros, 
              methodName = "DHARMa zero-inflation test", 
              ...)
}




#out1 <- capture.output(summary(T1))
#getAnywhere(print.summary.glmmTMB)
#getAnywhere(.prt.call.glmmTMB)
#getAnywhere(formComp)
#getAnywhere(trivialDisp)
#getAnywhere(.prt.family)
#getAnywhere(.prt.call.glmmTMB)
#getAnywhere(cat.f)
#getAnywhere(printDispersion)
#formComp <- function (object, type = "dispformula", target) {
#    ident(object$modelInfo$allForm[[type]], target) || ident(object$call[[type]], 
#        target)}
#trivialDisp <- function (object) { formComp(object, "dispformula", ~1) }
#glmmTMB:::.prt.call.glmmTMB(T1$call)

library(lme4)


MySummary.glmmTMB <- function (x, 
                               digits = max(3, getOption("digits") - 3), 
                               signif.stars = FALSE, 
                               ranef.comp = c("Variance", "Std.Dev."), 
                               show.resids = FALSE, 
                               ...) 
{
  .prt.family(x)
  #.prt.call.glmmTMB(x$call)
  cat("\n")
  .prt.aictab(x$AICtab)
  cat("\n")
  if (show.resids) 
    .prt.resids(x$residuals, digits = digits)
  if (any(whichRE <- !sapply(x$varcor, is.null))) {
    cat("Random effects:\n")
    for (nn in names(x$varcor[whichRE])) {
      cat("\n", glmmTMB:::cNames[[nn]], ":\n", sep = "")
      print(formatVC(x$varcor[[nn]], digits = digits, comp = ranef.comp), 
            quote = FALSE, digits = digits)
      .prt.grps(x$ngrps[[nn]], nobs = x$nobs)
    }
  }
  #if (trivialDisp(x)) {
  glmmTMB:::printDispersion(x$family, x$sigma)
  #}
  for (nn in names(x$coefficients)) {
    cc <- x$coefficients[[nn]]
    p <- length(cc)
    if (p > 0) {
      cat("\n", glmmTMB:::cNames[[nn]], ":\n", sep = "")
      printCoefmat(cc, zap.ind = 3, digits = digits, signif.stars = signif.stars)
    }
  }
  invisible(x)
}
